import copy

import torch
from torch import FloatTensor
from torch.nn import ReLU
from torch_geometric.explain.explanation import Explanation


class Adjust(object):
    def __init__(
        self,
        strategy: str = "rpn",
    ):
        self.strategy = strategy

    def forward(self, exp: Explanation) -> Explanation:
        exp_ = copy.copy(exp)
        _store = exp_.to_dict()
        for k, v in _store.items():
            if "mask" in k:
                for f_ in self.strategy:
                    if f_ == "r":
                        _store[k] = self.relu(v)
                    if f_ == "a":
                        _store[k] = self.absolute(v)
                    if f_ == "p":
                        if "edge" in k:
                            pass
                        else:
                            _store[k] = self.project(v)
                    if f_ == "n":
                        _store[k] = self.normalize(v)
            else:
                continue

        return exp_

    def relu(self, mask: FloatTensor) -> FloatTensor:
        relu = ReLU()
        mask_ = relu(mask)
        return mask_

    def normalize(self, mask: FloatTensor) -> FloatTensor:
        norm = torch.norm(mask, p="inf")
        if norm.item() > 0:
            mask_ = mask / norm.item()
            return mask_
        else:
            return mask

    def project(self, mask: FloatTensor) -> FloatTensor:
        if mask.ndim >= 2:
            mask_ = torch.sum(mask, dim=1)
            return mask_
        else:
            return mask

    def absolute(self, mask: FloatTensor) -> FloatTensor:
        mask_ = torch.abs(mask)
        return mask_
